#ifndef CORE_H
#define CORE_H
//
//  Core.h
//  Ice
//

#include <stdio.h>
#include <stdlib.h>

#include <boost/thread.hpp>
#include <boost/date_time.hpp>
#include <boost/interprocess/managed_shared_memory.hpp>
#include <boost/interprocess/ipc/message_queue.hpp>
#include <boost/interprocess/sync/interprocess_mutex.hpp>
#include <boost/interprocess/sync/sharable_lock.hpp>
#include <boost/interprocess/sync/upgradable_lock.hpp>
#include <boost/interprocess/sync/interprocess_upgradable_mutex.hpp>

//#include "Provehamus.h"
/*

 Our data:
 A. Exists in some coherant format.
 B. Have a layer of separate abstraction that is separate from the that determines the amount of power dedicated to sections of space.

   +------+------+
  /|     /|  *  /|
 +------+------+ |
 | +---*|-+----|-+
 |/|  * |/|  * |/|
 +------+------+ |
 | +-*--|-+-*--|-+
 |/   * |/*    |/
 +------+------+
 Figure 1.

 At its base level, each cube is going to look like Figure 2: A set of cubes that have such and such a quantity of particles, energy, and other such relevant properties. Our code will manipulate each cube's variables based upon surrounding space: a.k.a a standard n-body model at this point. However, n-body models can only go so far when one is dealing with such a large quantity of matter and space. This is where we introduce the statistical model, and incorporate it on a small scale.

   +------+------+
  /333333/111111/|
 +------+------+1|
 |333333|111111|1+
 |333333|111111|/|
 +------+------+7|
 |555555|777777|7+
 |555555|777777|/
 +------+------+
 Figure 2.

 A statistical model would rely in this situation upon more general descriptions, and can only produce approximate results; it's applied statistics rather than physics. However, it has the bonus of being far less taxing upon the processor, and allows one to work with far larger scenarios.
 This code will implement both. At its absolute smallest level, it will not be dealing with particles, but with densities of particles, and other such statistically manipulatable variables. It will resemble Figure 2 far more than Figure 1. One can, however, apply n-body simulation characteristics to this statistical model, through treating sectors of space as particles of cumulative mass, and using this to simplify the calculations.
 However, at this point we are left with an n-body simulation with statistical characteristics, one that will likely still consume too much power. Optimization will be a key component here in assuring a runable simulation. There are two
 */

#define memname "ice_plexor"
#define memcopyname "ice_plexor_copy"
#define gconst 0.0000000000667384
#define unit 1
#define time 100000//(seconds)

class vector
{
public:
	vector(){
		x = 0;
		y = 0;
		z = 0;};
	vector(int X, int Y, int Z)
	{
	x = X;
	y = Y;
	z = Z;
	};
	bool operator<(const vector& v) const
	{
		if(x<v.x)
			return -1;
		if(y<v.y)
			return -1;
		if(z<v.z)
			return -1;
		return 1;
	};
	bool operator!=(const vector& v) const
	{
		if(x<v.x)
			return 1;
		if(y<v.y)
			return 1;
		if(z<v.z)
			return 1;
		return -1;
	};
	double x,y,z;
};

//Core Functions
void initiateFusion();
void terminateFusion();

// Stellar Classes
//Dust is used here in place of a better word; it does not literally mean "dust," but rather a mass of some type.
struct dust
{
	std::string name;
	double mass;
	double charge;
};

class cube {
public:
    cube(){};
	cube(vector loc)
	{
	location = loc;
	};
	vector location;
	//List of pairs, of mediums and their total mass
	std::map<int,double> masses;

	double mass();
	double charge();

	vector direction;//Force in Newtons

	void addDust(int dustID, double mass);
	void removeDust(int dustID, double mass);
};

class conglomerate: public cube{
	
};

class dataplexor {
	public:
	//Simulation Et Cetera
	std::string simulationName;
	double cubeSize; //A side's length in meters

	vector size; //A side's length in Cubes
	std::map<vector,cube> cubeArray;
	std::vector<conglomerate> optArray;
	std::map<int,dust> dustArray;

	//Multithreading
	boost::interprocess::interprocess_upgradable_mutex mutex;
	
	cube* cubeAt(vector loc);
	cube* cubeAt(int x, int y, int z);

    bool halt;
};

class automaton {
    boost::thread m_thread;
	vector location;
public:
	automaton(vector loc);
	~automaton();
    void start();
    void join();
	bool timedJoin(int seconds);
    void callUponKronos(vector loc);
};
#endif CORE_H